package org.rexxla.bsf;

import java.io.*;

/** Invokes given Rexx or ooRexx program and supplies the commandline arguments.
 *
 * <pre>------------------------ Apache Version 2.0 license -------------------------
 *    Copyright (C) 2006 Rony G. Flatscher
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        <a href="http://www.apache.org/licenses/LICENSE-2.0">http://www.apache.org/licenses/LICENSE-2.0</a>
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 * ----------------------------------------------------------------------------- </pre>
 *
 * @author   Rony G. Flatscher
 * @version  1.02, 2006-01-28
 */

 /*
     last change: $Revision: 149 $ $Author: rony $ $Date: 2006-02-07 16:57:04 +0100 (Tue, 07 Feb 2006) $

     2006-01-28, ---rgf, fixed bug (arguments not passed on to Rexx), reporting the Rexx execution
                         error)

     2006-01-07, ---rgf, removed ability to look for com.ibm.bsf, if org.apache.bsf is not
                         found (to reduce confusion on error reports)
 */
public class RexxDispatcher {
        /** Version string indicating version of this class (majorVersion*100+minorVersion
         *  concatenated with a dot and the sorted date of last change.
         */
        static public String version = "102.20060128";

        /** Key for storing the received args-String array in the BSF registry for Rexx to
         *  get at it.
         */
        static private String keyName="allCommandLineArguments";

        private RexxDispatcher() {};   // do not allow to create an instance of this class

	/**
	 * Dispatches the Rexx/ooRexx program in args[0] and supplies it with the argument,
         * assembled by concatenating all remaining arguments with a blank. In addition the
         * String array <code>args[]</code> is saved in the BSF registry under the name
         * &quot;allCommandLineArguments&quot;.
	 *
	 * @param args arguments
	 *
	 */
	public static void main(String[] args)
        {
// System.err.println("args="+args.length);
            if (args.length == 0)
            {
                    System.err.println("usag:\n\tjava RexxDispatcher rexx_file_name [arguments]");
                    System.exit(1);
            }

            String [] args4rexx    = null;
            String    argline      = null;
            java.util.Vector vArgs = null;

            Object scriptString    = null;
            org.apache.bsf.BSFManager ap_mgr  = null;   // Apache's bsf, Java >= 1.3

            org.apache.bsf.BSFEngine  ap_rexx = null;   // Apache's bsf, Java >= 1.3

            try     // access to Apache's BSF there ?
            {
                ap_mgr  = new org.apache.bsf.BSFManager ();
                ap_mgr.declareBean(keyName, args, args.getClass());    // save args[] in registry
                ap_rexx = ap_mgr.loadScriptingEngine("rexx");
            }
            catch (Exception e)
            {
                e.printStackTrace();
                System.exit(1);
            }


                // if additional arguments, pass them as a blank delimited string to Rexx
            if (args.length>1)
            {
                vArgs=new java.util.Vector();
                argline="";
                boolean bEnquote=false;

                for (int i=1; i<args.length; i++)
                {
                    argline=argline + (argline.length()>0 ? " " : "");  // append a blank, if not first argument

                    // if an argument has a space re-enquote it!
                    bEnquote=false;
                    for (int k=0; k<args[i].length() && bEnquote==false; k++)
                    {
                        bEnquote=(args[i].charAt(k)==' ');
                    }
                    argline=argline + (bEnquote==true ? "\"" : "") + args[i] + (bEnquote==true ? "\"" : "");
                }
                vArgs.addElement(argline);      // add concatenated String
            }

            try
            {
                    // execute Rexx script
                Object obj =ap_rexx.apply( args[0],
                                        0,
                                        0,
                                        (Object) org.apache.bsf.util.IOUtils.getStringFromReader(new FileReader(args[0])),
                                        null,
                                        vArgs
                                      );
            }
            catch (Exception e)
            {
                System.err.println("\nException of type '"+e.getClass().getName()+"' thrown while invoking Rexx:\n"+e.getMessage());
                System.exit(1);
            }

            System.exit(0);
	}
}
